<?php
/**
 * FuzeWorks Component.
 *
 * The FuzeWorks PHP FrameWork
 *
 * Copyright (C) 2013-2019 TechFuze
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * @author    TechFuze
 * @copyright Copyright (c) 2013 - 2019, TechFuze. (http://techfuze.net)
 * @license   https://opensource.org/licenses/MIT MIT License
 *
 * @link  http://techfuze.net/fuzeworks
 * @since Version 1.2.0
 *
 * @version Version 1.2.0
 */

namespace FuzeWorks\Async\Constraint;
use FuzeWorks\Async\Constraint;
use FuzeWorks\Async\Task;
use FuzeWorks\Factory;

class GroupDependencyConstraint implements Constraint
{

    /**
     * @var string
     */
    protected $groupName;

    protected $returnStatus = Task::DELAYED;

    public function __construct(string $groupName)
    {
        $this->groupName = $groupName;
    }

    /**
     * @inheritDoc
     */
    public function intervene(Task $task, array $tasks): bool
    {
        // Find whether any task is not completed
        $hasFailed = false;
        $hasUnresolved = false;
        foreach ($tasks as $t)
        {
            foreach ($t->getConstraints() as $constraint)
            {
                // Check whether the constraint is a GroupConstraint
                if ($constraint instanceof GroupConstraint && $this->groupName === $constraint->getGroupName())
                {
                    // Check whether GroupConstraint has failed
                    if ($t->getStatus() === Task::CANCELLED)
                        $hasFailed = true;
                    // If the task is not cancelled and not completed, there are unresolved tasks
                    elseif ($t->getStatus() !== Task::COMPLETED)
                        $hasUnresolved = true;
                }
            }

            // If there are failed tasks, cancel this dependent
            if ($hasFailed)
            {
                $this->returnStatus = Task::CANCELLED;
                $task->setOutput('', 'Task cancelled due to failed group dependency.');
                return true;
            }
            elseif ($hasUnresolved)
            {
                $this->returnStatus = Task::DELAYED;
                return true;
            }
        }

        // Return the taskList
        //$task->setOutput(serialize($tasks), '');
        //$this->returnStatus = Task::CANCELLED;
        //return true;

        return false;
    }

    /**
     * @inheritDoc
     */
    public function blockCode(): int
    {
        return $this->returnStatus;
    }

    /**
     * @inheritDoc
     */
    public function delayTime(): int
    {
        return time() + 3;
    }
}